@@ -7,12 +7,13 @@
 // permissions and limitations relating to use of the SAFE Network Software.
 
 use crate::{
-    capacity::AdultsStorageInfo,
+    btree_set,
+    capacity::{Capacity, CHUNK_COPY_COUNT},
     error::convert_to_error_message,
     node_ops::{NodeDuties, NodeDuty},
     Error, Result,
 };
-use log::{debug, info, warn};
+use log::{info, warn};
 use sn_data_types::{Blob, BlobAddress, PublicKey};
 use sn_messaging::{
     client::{BlobDataExchange, BlobRead, BlobWrite, ClientSigned, CmdError, QueryResponse},
@@ -28,70 +29,39 @@ use std::{
 use xor_name::XorName;
 
 use super::{
-    adult_liveness::AdultLiveness, adult_reader::AdultReader, build_client_error_response,
-    build_client_query_response,
+    adult_liveness::AdultLiveness, build_client_error_response, build_client_query_response,
 };
 
-// The number of separate copies of a blob chunk which should be maintained.
-pub(crate) const CHUNK_COPY_COUNT: usize = 4;
-
 /// Operations over the data type Blob.
 pub(super) struct BlobRecords {
-    adult_storage_info: AdultsStorageInfo,
-    reader: AdultReader,
+    capacity: Capacity,
     adult_liveness: AdultLiveness,
 }
 
 impl BlobRecords {
-    pub(super) fn new(adult_storage_info: AdultsStorageInfo, reader: AdultReader) -> Self {
+    pub(super) fn new(capacity: Capacity) -> Self {
         Self {
-            adult_storage_info,
-            reader,
+            capacity,
             adult_liveness: AdultLiveness::new(),
         }
     }
 
     pub async fn get_data_of(&self, prefix: Prefix) -> BlobDataExchange {
         // Prepare full_adult details
-        let full_adults = self
-            .adult_storage_info
-            .full_adults
-            .read()
-            .await
-            .iter()
-            .filter(|name| prefix.matches(name))
-            .copied()
-            .collect();
+        let full_adults = self.capacity.full_adults_matching(prefix).await;
         BlobDataExchange { full_adults }
     }
 
-    pub async fn update(&self, blob_data: BlobDataExchange) -> Result<()> {
-        debug!("Updating Blob records");
-        let mut orig_full_adults = self.adult_storage_info.full_adults.write().await;
-
+    pub async fn update(&self, blob_data: BlobDataExchange) {
         let BlobDataExchange { full_adults } = blob_data;
-
-        for adult in full_adults {
-            let _ = orig_full_adults.insert(adult);
-        }
-
-        Ok(())
+        self.capacity.insert_full_adults(full_adults).await
     }
 
     /// Registered holders not present in provided list of members
     /// will be removed from adult_storage_info and no longer tracked for liveness.
     pub async fn retain_members_only(&mut self, members: BTreeSet<XorName>) -> Result<()> {
         // full adults
-        let mut full_adults = self.adult_storage_info.full_adults.write().await;
-        let absent_adults = full_adults
-            .iter()
-            .filter(|key| !members.contains(key))
-            .cloned()
-            .collect::<Vec<_>>();
-
-        for adult in &absent_adults {
-            let _ = full_adults.remove(adult);
-        }
+        self.capacity.retain_members_only(&members).await;
 
         // stop tracking liveness of absent holders
         self.adult_liveness.retain_members_only(members);
@@ -114,44 +84,28 @@ impl BlobRecords {
     }
 
     /// Adds a given node to the list of full nodes.
-    pub async fn increase_full_node_count(&mut self, node_id: PublicKey) -> Result<()> {
-        info!("No. of Full Nodes: {:?}", self.full_nodes().await);
-        info!("Increasing full_node count");
-        let _ = self
-            .adult_storage_info
-            .full_adults
-            .write()
-            .await
-            .insert(XorName::from(node_id));
-        Ok(())
+    pub async fn increase_full_node_count(&mut self, node_id: PublicKey) {
+        info!(
+            "No. of full Adults: {:?}",
+            self.capacity.full_adults_count().await
+        );
+        info!("Increasing full Adults count");
+        self.capacity
+            .insert_full_adults(btree_set!(XorName::from(node_id)))
+            .await;
     }
 
     /// Removes a given node from the list of full nodes.
     #[allow(unused)] // TODO: Remove node from full list at 50% ?
-    async fn decrease_full_node_count_if_present(&mut self, node_name: XorName) -> Result<()> {
-        info!("No. of Full Nodes: {:?}", self.full_nodes().await);
+    async fn decrease_full_adults_count_if_present(&mut self, node_name: XorName) {
+        info!(
+            "No. of Full Nodes: {:?}",
+            self.capacity.full_adults_count().await
+        );
         info!("Checking if {:?} is present as full_node", node_name);
-        match self
-            .adult_storage_info
-            .full_adults
-            .write()
-            .await
-            .remove(&node_name)
-        {
-            true => {
-                info!("Node present in DB, remove successful");
-                Ok(())
-            }
-            false => {
-                info!("Node not found on full_nodes db");
-                Ok(())
-            }
-        }
-    }
-
-    /// Number of full chunk storing nodes in the section.
-    async fn full_nodes(&self) -> u8 {
-        self.adult_storage_info.full_adults.read().await.len() as u8
+        self.capacity
+            .remove_full_adults(btree_set!(node_name))
+            .await;
     }
 
     async fn send_chunks_to_adults(
@@ -161,19 +115,14 @@ impl BlobRecords {
         client_signed: ClientSigned,
         origin: EndUser,
     ) -> Result<NodeDuty> {
-        let target_holders = self
-            .get_holders_for_chunk(data.name())
-            .await
-            .iter()
-            .cloned()
-            .collect::<BTreeSet<_>>();
+        let target_holders = self.capacity.get_chunk_holder_adults(data.name()).await;
 
         info!("Storing {} copies of the data", target_holders.len());
 
-        if target_holders.is_empty() {
+        if CHUNK_COPY_COUNT > target_holders.len() {
             return self
                 .send_error(
-                    Error::NoAdults(self.reader.our_prefix().await),
+                    Error::NoAdults(self.capacity.our_prefix().await),
                     msg_id,
                     origin,
                 )
@@ -229,9 +178,8 @@ impl BlobRecords {
             &src,
             response.is_success(),
         ) {
-            let full_adults = self.adult_storage_info.full_adults.read().await;
             // If a full adult responds with error. Drop the response
-            if full_adults.contains(&src) && !response.is_success() {
+            if !response.is_success() && self.capacity.is_full(&src).await {
                 // We've already responded already with a success
                 // so do nothing
             } else {
@@ -277,8 +225,7 @@ impl BlobRecords {
         client_signed: ClientSigned,
         origin: EndUser,
     ) -> Result<NodeDuty> {
-        let targets = self.get_holders_for_chunk(address.name()).await;
-        let targets = targets.iter().cloned().collect::<BTreeSet<_>>();
+        let targets = self.capacity.get_chunk_holder_adults(address.name()).await;
 
         let msg = NodeMsg::NodeCmd {
             cmd: NodeCmd::Chunks {
@@ -297,13 +244,7 @@ impl BlobRecords {
 
     pub(super) async fn republish_chunk(&mut self, data: Blob) -> Result<NodeDuty> {
         let owner = data.owner();
-
-        let target_holders = self
-            .get_holders_for_chunk(data.name())
-            .await
-            .into_iter()
-            .collect::<BTreeSet<_>>();
-
+        let target_holders = self.capacity.get_chunk_holder_adults(data.name()).await;
         // deterministic msg id for aggregation
         let msg_id = MessageId::from_content(&(*data.name(), owner, &target_holders))?;
 
@@ -341,18 +282,12 @@ impl BlobRecords {
         msg_id: MessageId,
         origin: EndUser,
     ) -> Result<NodeDuty> {
-        let targets = self.get_holders_for_chunk(address.name()).await;
-        let full_adults = self.adult_storage_info.full_adults.read().await.clone();
-
-        let targets = targets
-            .into_iter()
-            .chain(full_adults.into_iter())
-            .collect::<BTreeSet<_>>();
+        let targets = self.capacity.get_chunk_holder_adults(address.name()).await;
 
         if targets.is_empty() {
             return self
                 .send_error(
-                    Error::NoAdults(self.reader.our_prefix().await),
+                    Error::NoAdults(self.capacity.our_prefix().await),
                     msg_id,
                     origin,
                 )
@@ -384,24 +319,15 @@ impl BlobRecords {
             Ok(NodeDuty::NoOp)
         }
     }
-
-    // Returns `XorName`s of the target holders for an Blob chunk.
-    // Used to fetch the list of holders for a new chunk.
-    async fn get_holders_for_chunk(&self, target: &XorName) -> Vec<XorName> {
-        let full_adults = self.adult_storage_info.full_adults.read().await;
-        self.reader
-            .non_full_adults_closest_to(&target, &full_adults, CHUNK_COPY_COUNT)
-            .await
-    }
 }
 
 fn validate_data_owner(data: &Blob, requester: &PublicKey) -> Result<()> {
     if data.is_private() {
         data.owner()
-            .ok_or_else(|| Error::InvalidOwners(*requester))
+            .ok_or_else(|| Error::InvalidOwner(*requester))
             .and_then(|data_owner| {
                 if data_owner != requester {
-                    Err(Error::InvalidOwners(*requester))
+                    Err(Error::InvalidOwner(*requester))
                 } else {
                     Ok(())
                 }
